#define HAVE_TIMESPEC
#include <videosystem3/videosystem3.h>

/* Note: in order for this example to work properly,

   b) a syslist file needs to exist
      ( variable syslistFileNamePath in syslistExample() )

   a) a proper video server with avine output interface (e.g. SGP, 
      Raw2Jpeg) needs to be alive and have fresh images available
      ( variable tineAddress in videoImagesExample() )

   Stefan Weisse (SW), Apr 29, 2013
*/

void videoImagesExample()
{
    int numFrames = 10;
    bool bSaveDataToFile = true;
    const char *fileName = "sequence.imc";
 
    //const char *tineAddress = "/PITZ/FG4S.Prosilica/Output";
    //const char *tineAddress = "/PITZ/FG5S.Analogue/Output";
    const char *tineAddress = "/PITZ/TV1S.Pros.1/Output";

    //const char *tinePropName = "Frame";
    const char *tinePropName = "Frame.Sched";
        
    VSBUNCHOFIMAGES seq;

    const unsigned int buflenconst = 150;
    unsigned int buflen=buflenconst;
    char buffer[buflenconst];
    unsigned int vMajor, vMinor, vSubMinor;

    int reti = vsSetVerbose( 2 ); // 2: verbose to stdout
    if (reti != 1) printf("vsSetVerbose( 2 ) failed with return code %d!\n", reti);


    printf("\nLibrary info:\n");
    char *retc = vsLibraryInfoEx( &vMajor, &vMinor, &vSubMinor, buffer, &buflen);
    if (retc == NULL)
        printf(" vsLibraryInfo returned NULL! (vX.Y=%u.%u.%u)\n\n",vMajor, vMinor, vSubMinor);
    else
        printf(" vsLibraryInfo returned \"%s\" (vX.Y=%u.%u.%u)\n\n", retc, vMajor, vMinor, vSubMinor);


    printf("Grabbing %d frames from '%s[%s]'...", numFrames, tineAddress, tinePropName);

    vsInitializeBunchStructure( &seq );

    int iError=0;
    int iTineError=0;

    int iBreak = 0; // breaking requires separate thread which sets this variable
    double dPercentageFinished=0.0;

    int ret = vsGrabFramesEx(   &seq, 
                                tineAddress, 
                                tinePropName, 
                                numFrames, 
                                1000, 
                                TDC_CM_TIMER|TDC_CM_SYNCNOTIFY|TDC_CM_CONNECT,
                                &iError,
                                &iTineError,
                                &iBreak,
                                &dPercentageFinished);

    printf("dPercentageFinished=%f\n", dPercentageFinished);

    if (ret > 0)
    {
        printf("ok.\n");
        if (iError != 0) 
            printf("warning: vsGrabFramesEx returned error condition: iError=%d iTineError=%d\n", iError, iTineError);
        if (ret < numFrames) 
            printf("warning: only %d out of %d frames have been grabbed.\n", ret, numFrames);

        int isSequence = vsIsSequence(&seq);
        if (isSequence != 1)
            printf("vsIsSequence(&seq) returned %d (-1 err, 0 no sequence)\n", isSequence);
        else // if (isSequence == 1)
        {
            printf("Bunch of frames is a sequence.\n");
            int isCompleteSequence = vsIsCompleteSequence(&seq);
            if (isCompleteSequence != 1)
                printf("vsIsCompleteSequence: %d (-1 err, 0 no)\n", isCompleteSequence);
            else 
                printf("Bunch of frames is a complete sequence.\n");

            if (bSaveDataToFile == true)
            {
                printf("\ntrying to decompress image sequence...");

                int ret1 = vsDecompressSequence( &seq, 1 );
                if (ret1 > 0)
                {
                    VIDINFHDR vidH;
                    printf("ok.\n");

                    printf("\nconverting to vidinfhdr...");
                    int iBunchUntouched=1;
                    int iLMC=1; // low memory consumtion at the cost of destroying &seq
                    ret1 = vsBunchToVIDINFHDREx( &seq, &vidH, iLMC, &iBunchUntouched );
                    if (ret1 == 1)
                    {
                        printf("ok.\n");

                        int iBreak = 0;
                        double dPercFin = 0.0;

                        printf("\nsaving to file '%s'...", fileName);
                        //iBreak and dPercFin are rather useless here
                        INT32 retv2 = save_imc_fileEx( fileName, &vidH, &iBreak, &dPercFin); 
                        if (retv2 != 0) 
                            printf("\nError saving v2 header to file. Error code: %d\n", retv2);
                        else 
                            printf("ok (percent finished: %f).\n", dPercFin);
            
                        free_vidinfhdr( &vidH );
                    }
                    else
                        printf("\nError converting to v2 vidinfhdr structure (error code: %d)\n", ret1);
                }
                else 
                    printf("\nError decompressing image sequence (error code: %d).\n", ret1);
            }
        }
    }
    else 
        printf("Not ok. ret=%d iError=%d iTineError=%d\n", ret, iError, iTineError);


    vsFreeBunch( &seq );

    printf("videoImagexExample: At the end.\n");

}


// char *returnPrintableNAME64( NAME64 &aArgument ) - helper function for syslistExample()
char *returnPrintableNAME64( NAME64 &aArgument )
{
    static char internalStringBuffer[65];

    platformsnprintf(internalStringBuffer, sizeof(internalStringBuffer)-1, "%s", aArgument.name);
    internalStringBuffer[sizeof(internalStringBuffer)-1] = 0;

    return internalStringBuffer;
}

// char *returnPrintableNAME64I( NAME64I &aArgument ) - helper function for syslistExample()
char *returnPrintableNAME64I( NAME64I &aArgument )
{
    static char internalStringBuffer[65];

    platformsnprintf(internalStringBuffer, sizeof(internalStringBuffer)-1, "%s", aArgument.name);
    internalStringBuffer[sizeof(internalStringBuffer)-1] = 0;

    return internalStringBuffer;
}

void syslistExample()
{
    const unsigned int cNumServerNames = 100;
    NAME64 tineContext;
    NAME64 tineServerNames[cNumServerNames];
    unsigned int numServerNames = cNumServerNames;
    const char syslistFileNamePath[] = "PITZsyslist.xml";

    int ret = vsGetAllRawVideoServerNames(  syslistFileNamePath, 
                                            &tineContext, 
                                            &tineServerNames[0], 
                                            &numServerNames );


    printf("vsGetAllRawVideoServerNames returned: %d (1 -> success, <1 -> failure)\n", ret);

    if (ret == 1)
    {
        printf("TINE Context: %s\n", returnPrintableNAME64(tineContext));

        printf("%u Raw Video Server Names returned.\n", numServerNames);

        if (numServerNames >0)
        {
            for (unsigned int i=0;i<numServerNames;i++)
                printf("Server #%03d: '%s'\n", i+1, returnPrintableNAME64(tineServerNames[i]) );
        }
    }

    
    const unsigned int cNumCameraNames = 100;
    NAME64 cameraNames[cNumCameraNames];
    NAME64I cameraNameIDs[cNumCameraNames];

    unsigned int numCameraNames = cNumCameraNames;

    ret = vsGetAllImageSourceNames( "PITZsyslist.xml", 
                                    &cameraNames[0], 
                                    NULL,
                                    &numCameraNames );


    printf("vsGetAllImageSourceNames (NAME64) returned: %d (1 -> success, <1 -> failure)\n", ret);

    if (ret == 1)
    {
        printf("%u Image Source Names returned.\n", numCameraNames);

        if (numCameraNames >0)
        {
            for (unsigned int i=0;i<numCameraNames;i++)
                printf("Image Source #%03d: '%s'\n", i+1, 
                                                     returnPrintableNAME64(cameraNames[i]) );
        }
    }

    numCameraNames = cNumCameraNames;

    ret = vsGetAllImageSourceNames( "PITZsyslist.xml", 
                                    NULL, 
                                    &cameraNameIDs[0],
                                    &numCameraNames );

    printf("vsGetAllImageSourceNames (NAME64I) returned: %d (1 -> success, <1 -> failure)\n", ret);

    if (ret == 1)
    {
        printf("%u Image Source Names returned.\n", numCameraNames);

        if (numCameraNames >0)
        {
            for (unsigned int i=0;i<numCameraNames;i++)
                printf("Image Source #%03d: '%s'=%d\n", i+1, 
                                                        returnPrintableNAME64I(cameraNameIDs[i]),
                                                        cameraNameIDs[i].ival);
        }
    }
    
    // down from here, one could implement the following:
    // - find out whether server N is alive
    // - find out which camera is currently connected at server N

}



int main( int argc, const char *argv[])
{
    videoImagesExample();

    syslistExample();
    
    return 0;
}
