#ifndef __PLATFORM_H_INCLUDED_SW__
#define __PLATFORM_H_INCLUDED_SW__

/** \file platform.h
 * \brief Declares platform-specifics.

    \author Stefan Weisse / DESY 
    \date   2008
    
    Does neccessary platform-based includes, definitions, data type typedefs, platform-based structure typedefs 
    and forward declaration for platform-wrapped functions. Works in conjunction with platform.cpp, which does 
    necessary definition of function based on defines done in here.
 
    In order to tie to a special platform, a general include file named "system.h" needs to be created inside the
    project. These file should not include much, but just \#define the desired platform type and \#undef all other 
    defined platform types. It is needed for proper branching inside platform.h.

    For abstract data types, some typedefs were invented:
    INT32		signed 32bit integer value
    UINT32		unsigned 32bit integer value

    DOUBLE		double precision float value (64 bits in size)
    FLOAT		single precision float value (32 bits in size)

    INT8		signed 8bit integer value
    UINT8		unsigned 8bit integer value

    INT16		signed 16bit integer value
    UINT16		unsigned 16bit integer value

    INT64		signed 64bit integer value
    UINT64		unsigned 64bit integer value

    HANDLE		pointer to memory location (e.g. mapped to void *)

    In addition to that, certain defines and typedefs will be made for
    thread creation compatibility, for function-mapping like snprintf and for
    path separator(s) and wildcards in platform-dependent manner which is needed 
    for configuration file access, naming lookup and support.
*/


// safety checking outer defines (only one of them can be defined)
// SW Dec 13, 2011 outer defines
// SW Dec 13, 2011 schliessen sich gegenseitig aus
// SW Dec 13, 2011 SYSTEM_LINUX
// SW Dec 13, 2011 SYSTEM_WINDOWS_VC6
// SW Dec 13, 2011 SYSTEM_WINDOWS_VS2010_X86
// SW Dec 13, 2011 SYSTEM_WINDOWS_VS2010_X64
// SW Oct 2, 2017  SYSTEM_WINDOWS_VS2015_X64
// SW Oct 2, 2017  SYSTEM_WINDOWS_VS2015_X86
// SW Oct 12, 2021  SYSTEM_WINDOWS_VS2019_X64
// SW Oct 12, 2021  SYSTEM_WINDOWS_VS2019_X86

#if defined(SYSTEM_WINDOWS_VS2005)
    #error SYSTEM_WINDOWS_VS2005 no longer supported as of / was removed on Oct 2, 2017!
#endif

#if defined(SYSTEM_WINDOWS_VS2010_X86)
    #error SYSTEM_WINDOWS_VS2010_X86 no longer supported as of / was removed on Oct 12, 2021!
#endif

#if defined(SYSTEM_WINDOWS_VS2010_X64)
    #error SYSTEM_WINDOWS_VS2010_X64 no longer supported as of / was removed on Oct 12, 2021!
#endif

#if defined(SYSTEM_SOLARIS)
    #error SYSTEM_SOLARIS no longer supported as of / was removed on Oct 12, 2021!
#endif

#if defined(SYSTEM_WINDOWS_VC6)
    #error SYSTEM_WINDOWS_VC6 no longer supported as of / was removed on Oct 12, 2021!
#endif


#if defined(SYSTEM_LINUX)

    #if defined(SYSTEM_WINDOWS_VS2015_X86)
        #error SYSTEM_WINDOWS_VS2015_X86 and SYSTEM_LINUX are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2015_X64)
        #error SYSTEM_WINDOWS_VS2015_X64 and SYSTEM_LINUX are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X86)
        #error SYSTEM_WINDOWS_VS2019_X86 and SYSTEM_LINUX are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X64)
        #error SYSTEM_WINDOWS_VS2019_X64 and SYSTEM_LINUX are mutually exclusive!
    #endif

#elif defined(SYSTEM_WINDOWS_VS2015_X86)

    #if defined(SYSTEM_LINUX)
        #error SYSTEM_LINUX and SYSTEM_WINDOWS_VS2015_X86 are mutually exclusive!
    #endif 

    #if defined(SYSTEM_WINDOWS_VS2015_X64)
        #error SYSTEM_WINDOWS_VS2015_X64 and SYSTEM_WINDOWS_VS2015_X86 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X86)
        #error SYSTEM_WINDOWS_VS2019_X86 and SYSTEM_WINDOWS_VS2015_X86 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X64)
        #error SYSTEM_WINDOWS_VS2019_X64 and SYSTEM_WINDOWS_VS2015_X86 are mutually exclusive!
    #endif


#elif defined(SYSTEM_WINDOWS_VS2015_X64)

    #if defined(SYSTEM_LINUX)
        #error SYSTEM_LINUX and SYSTEM_WINDOWS_VS2015_X64 are mutually exclusive!
    #endif 

    #if defined(SYSTEM_WINDOWS_VS2015_X86)
        #error SYSTEM_WINDOWS_VS2015_X86 and SYSTEM_WINDOWS_VS2015_X64 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X86)
        #error SYSTEM_WINDOWS_VS2019_X86 and SYSTEM_WINDOWS_VS2015_X64 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X64)
        #error SYSTEM_WINDOWS_VS2019_X64 and SYSTEM_WINDOWS_VS2015_X64 are mutually exclusive!
    #endif

#elif defined(SYSTEM_WINDOWS_VS2019_X86)

    #if defined(SYSTEM_LINUX)
        #error SYSTEM_LINUX and SYSTEM_WINDOWS_VS2019_X86 are mutually exclusive!
    #endif 

    #if defined(SYSTEM_WINDOWS_VS2015_X86)
        #error SYSTEM_WINDOWS_VS2015_X86 and SYSTEM_WINDOWS_VS2019_X86 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2015_X64)
        #error SYSTEM_WINDOWS_VS2015_X64 and SYSTEM_WINDOWS_VS2019_X86 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X64)
        #error SYSTEM_WINDOWS_VS2019_X64 and SYSTEM_WINDOWS_VS2019_X86 are mutually exclusive!
    #endif

#elif defined(SYSTEM_WINDOWS_VS2019_X64)

    #if defined(SYSTEM_LINUX)
        #error SYSTEM_LINUX and SYSTEM_WINDOWS_VS2015_X64 are mutually exclusive!
    #endif 

    #if defined(SYSTEM_WINDOWS_VS2015_X86)
        #error SYSTEM_WINDOWS_VS2015_X86 and SYSTEM_WINDOWS_VS2019_X64 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2015_X64)
        #error SYSTEM_WINDOWS_VS2015_X64 and SYSTEM_WINDOWS_VS2019_X64 are mutually exclusive!
    #endif

    #if defined(SYSTEM_WINDOWS_VS2019_X86)
        #error SYSTEM_WINDOWS_VS2019_X86 and SYSTEM_WINDOWS_VS2019_X64 are mutually exclusive!
    #endif

#endif

// SW Jul 28, 2014 START EXPORTED DEFINES: the following defines are necessary 
//                       in case internal declarations of platform.h are not exported 
//                       (hidden for external use of videosystem3.h of ClnAPI)
// 
#ifdef SYSTEM_LINUX
    #define  PLF_SYSTEM_LINUX
#endif

#ifdef SYSTEM_WINDOWS_VS2015_X86
    #define  PLF_SYSTEM_WIN2015 
    #define  PLF_SYSTEM_WIN2015_X86 
#endif

#ifdef SYSTEM_WINDOWS_VS2015_X64
    #define  PLF_SYSTEM_WIN2015 
    #define  PLF_SYSTEM_WIN2015_X64 
#endif

#ifdef SYSTEM_WINDOWS_VS2019_X86
    #define  PLF_SYSTEM_WIN2019 
    #define  PLF_SYSTEM_WIN2019_X86 
#endif

#ifdef SYSTEM_WINDOWS_VS2019_X64
    #define  PLF_SYSTEM_WIN2019 
    #define  PLF_SYSTEM_WIN2019_X64 
#endif


// SW Jul 28, 2014 END EXPORTED DEFINES

//defines necessary for videosystem3 c library (at some places DOUBLE and INT32 are used) START
#ifndef INT32
    typedef		int				INT32;
#endif

#ifndef DOUBLE
    typedef		double			DOUBLE;
#endif

#if defined(PLF_SYSTEM_LINUX) 
    #define  platformsnprintf snprintf
    //#define platformstrncpy(sDestBffer, sizeOfBuffer, csSourceBuffer, count) strncpy(sDestBuffer, csSourceBuffer, count)
    //#define platformstrcpy(sDestBffer, sizeOfBuffer, csSourceBuffer) strcpy(sDestBuffer, csSourceBuffer)
    #define platformstrtok(sInput, sDelimiter, sContext) strtok_r(sInput, sDelimiter, sContext)
#endif

#if defined(PLF_SYSTEM_WIN2015) || defined(PLF_SYSTEM_WIN2019)
    #define  platformsnprintf _snprintf
    //#define platformstrncpy(sDestBffer, sizeOfBuffer, csSourceBuffer, count) strncpy_s(sDestBffer, sizeOfBuffer, csSourceBuffer, count)
    //#define platformstrcpy(sDestBffer, sizeOfBuffer, csSourceBuffer) strcpy_s(sDestBffer, sizeOfBuffer, csSourceBuffer)
    #define platformstrtok(sInput, sDelimiter, sContext) strtok_s(sInput, sDelimiter, sContext)

#endif

// for "extern bool platformFileSeek64(FILE *apFile, int64_t ai64Offset, int aiOrigin); // introduced 2022-05-05"'s parameter aiOrigin
#define PLF_FILE_SEEK_SET SEEK_SET
#define PLF_FILE_SEEK_CUR SEEK_CUR
#define PLF_FILE_SEEK_END SEEK_END

//defines necessary for videosystem3 c library (at some places DOUBLE and INT32 are used) END

#ifndef VIDEOSYSTEM3_PLATFORM_H_EXTERNAL // SW Jul 28, 2014 VIDEOSYSTEM3_PLATFORM_H_EXTERNAL added

#ifdef SYSTEM_LINUX
    #include <stdio.h>	
    #include <stdlib.h>
    #include <errno.h>
    #include <string.h>
    #include <ctype.h>
    #include <termios.h>
    #include <assert.h>
    #include <unistd.h>
    #include <fcntl.h>
    #include <pwd.h> // SW pwd.h added Sept 3, 2013 due to getpwuid() platformGetUserName
  
    // SW Nov 27, 2012 1/3 added limits.h because gcc 4.4.6 on amd64_rhel60 
    // SW Nov 27, 2012 2/3 (picus4-vm2) errors like LONG_MAX not declared 
    // SW Nov 27, 2012 3/3 in this scope (usefulfunctions.cpp::Str2Int())
    #include <limits.h>	

    #define __STDC_FORMAT_MACROS // SW added Jan 8, 2015 according to http://stackoverflow.com/questions/26182336/priuptr-preprocessor-bug-in-gcc
    #include <inttypes.h> // SW added Jan 5, 2015 for platform independent widely compatible printf identifiers

    #include <sys/timeb.h>
    #include <sys/times.h>
    #include <sys/types.h>
    #include <sys/stat.h>

    #include <netdb.h>
    #include <pthread.h> 
    #include <arpa/inet.h>
    #define  PLF_USING_PTHREAD

    #define  SW_PLATFORM_STRING "Linux"

    #define  PLF_UNIX_TCPIPSTACK

    #define  PLF_PATH_SEPARATOR_CHAR '/'
    #define  PLF_PATH_SEPARATOR_CHAR_AS_STRING "/"
    #define  PLF_ALLFILES_WILDCARD	"*"
    #define  MAX_STRING_LEN		1024
    #define  platformvsnprintf vsnprintf
//	#define  platformsnprintf snprintf
    // platformstrcasecmp case insensitive string comparison
    #define  platformstrcasecmp strcasecmp 
    
    #define  platformstrncasecmp strncasecmp // only compare N (rightmost argument) number of leftmost characters (insensitive match of some left part of string)
    
    #define  PLF_MAX_FDSETS 		64
    
    //SW disabled Oct 16, 2012 #undef PLF_ENDIANCHANGE // SW! not properly implemented

    #define PLF_LITTLE_ENDIAN
#endif



#ifdef SYSTEM_WINDOWS_VS2015_X86
    #define NOMINMAX // for usefulfunctions.cpp std::numeric_limits<float>::max() and ::min(), otherwise
                     // collided with 

    #ifndef _USE_32BIT_TIME_T
        #define  _USE_32BIT_TIME_T
    #endif

    #include <stdio.h>
    #define WIN32_LEAN_AND_MEAN  // SW Dec 21, 2015 added so that winsock does not get included by windows.h (will collide with tine v4.5+ (year 2015) project.def
    // which may include winsock2 headers (IPV6)
    #include <windows.h>
    #include <WinSock2.h>    // SW Dec 21, 2015 added 
    #include <conio.h>
    #include <time.h>
    #include <assert.h>
    #include <sys\stat.h>
    #include <sys\timeb.h>
    
    #define __STDC_FORMAT_MACROS  
    #include <inttypes.h> // for PRIxPTR, PRIuPTR, PRIx64...

    //#pragma warning(disable : 4996) // disable deprecated warnings SW commented Oct 15, 2019

    #define  SW_PLATFORM_STRING "win32(msvs2015,x86)"

    #define  PLF_WINNT_TCPIPSTACK
    #define  PLF_TCPIPSTACK_VS2015_OR_GREATER // for deprecated gethostbyname -> getaddrinfo
    
    #define  PLF_PATH_SEPARATOR_CHAR '\\'
    #define  PLF_PATH_SEPARATOR_CHAR_AS_STRING "\\"	
    #define  PLF_ALLFILES_WILDCARD	"*.*"	
    #define  platformvsnprintf _vsnprintf
    
    // platformstrcasecmp case insensitive string comparison
    #define  platformstrcasecmp _stricmp
    #define  platformstrncasecmp _strnicmp // only compare N (rightmost argument) characters (insensitive match of some left part of string)

    #define  PLF_MAX_FDSETS 		FD_SETSIZE

    #define PLF_LITTLE_ENDIAN
    #ifndef HAVE_TIMESPEC
        #define HAVE_TIMESPEC // SW Jan 5, 2018 for tine srvcore.h to avoid error timespec struct type redefinition windows headers 
        // (C:\Program Files (x86)\Windows Kits\10\Include\10.0.10240.0\ucrt\time.h(39)) vs. tine (at least v4.6.0)
    #endif


#endif

#if defined(SYSTEM_WINDOWS_VS2015_X64)
    #define NOMINMAX // for usefulfunctions.cpp std::numeric_limits<float>::max() and ::min() (from 
                     // #include <limits>), otherwise
                     // collides with c:\Program Files (x86)\Windows Kits\8.1\Include\shared\minwindef.h 
                     // definition of macros min and max)

    #include <stdio.h>

    #define WIN32_LEAN_AND_MEAN  // SW Dec 21, 2015 added so that winsock does not get included by windows.h (will collide with tine v4.5+ (year 2015) project.def
    // which may include winsock2 headers (IPV6)
    #include <windows.h>
    #include <WinSock2.h>    // SW Dec 21, 2015 added 
    #include <conio.h>
    #include <time.h>
    #include <assert.h>
    #include <sys\stat.h>
    #include <sys\timeb.h>

    #define __STDC_FORMAT_MACROS
    #include <inttypes.h>

    //#pragma warning(disable : 4996) // disable deprecated warnings SW commented Oct 15, 2019

    #define  SW_PLATFORM_STRING "win64(msvs2015,x64)"

    #define  PLF_TCPIPSTACK_VS2015_OR_GREATER // for deprecated gethostbyname -> getaddrinfo
    #define  PLF_WINNT_TCPIPSTACK

    #define  PLF_PATH_SEPARATOR_CHAR '\\'
    #define  PLF_PATH_SEPARATOR_CHAR_AS_STRING "\\"	
    #define  PLF_ALLFILES_WILDCARD	"*.*"	
    #define  platformvsnprintf _vsnprintf
    // platformstrcasecmp case insensitive string comparison
    #define  platformstrcasecmp _stricmp
    #define  platformstrncasecmp _strnicmp // only compare N (rightmost argument) characters (insensitive match of some left part of string)

    #define  PLF_MAX_FDSETS 		FD_SETSIZE

    #define PLF_LITTLE_ENDIAN

    #define HAVE_TIMESPEC // SW Jan 5, 2018 for tine srvcore.h to avoid error timespec struct type redefinition windows headers 
    // (C:\Program Files (x86)\Windows Kits\10\Include\10.0.10240.0\ucrt\time.h(39)) vs. tine (at least v4.6.0)

#endif

#ifdef SYSTEM_WINDOWS_VS2019_X86
    #define NOMINMAX // for usefulfunctions.cpp std::numeric_limits<float>::max() and ::min() (from 
        // #include <limits>), otherwise
        // collides with c:\Program Files (x86)\Windows Kits\8.1\Include\shared\minwindef.h 
        // definition of macros min and max)

    #ifndef _USE_32BIT_TIME_T
        #define  _USE_32BIT_TIME_T
    #endif

    #include <stdio.h>
    #define WIN32_LEAN_AND_MEAN  // SW Dec 21, 2015 added so that winsock does not get included by windows.h (will collide with tine v4.5+ (year 2015) project.def
    // which may include winsock2 headers (IPV6)
    #include <windows.h>
    #include <WinSock2.h>    // SW Dec 21, 2015 added 
    #include <conio.h>
    #include <time.h>
    #include <assert.h>
    #include <sys\stat.h>
    #include <sys\timeb.h>

    #define __STDC_FORMAT_MACROS  
    #include <inttypes.h> // for PRIxPTR, PRIuPTR, PRIx64...

    //#pragma warning(disable : 4996) // disable deprecated warnings SW commented Oct 15, 2019

    #define  SW_PLATFORM_STRING "win32(msvs2019,x86)"
    
    #define  PLF_WINNT_TCPIPSTACK
    #define  PLF_TCPIPSTACK_VS2015_OR_GREATER // for deprecated gethostbyname -> getaddrinfo
    
    #define  PLF_PATH_SEPARATOR_CHAR '\\'
    #define  PLF_PATH_SEPARATOR_CHAR_AS_STRING "\\"	
    #define  PLF_ALLFILES_WILDCARD	"*.*"	
    #define  platformvsnprintf _vsnprintf

    // platformstrcasecmp case insensitive string comparison
    #define  platformstrcasecmp _stricmp
    #define  platformstrncasecmp _strnicmp // only compare N (rightmost argument) characters (insensitive match of some left part of string)

    #define  PLF_MAX_FDSETS 		FD_SETSIZE

    #define PLF_LITTLE_ENDIAN
    #ifndef HAVE_TIMESPEC
        #define HAVE_TIMESPEC // SW Jan 5, 2018 for tine srvcore.h to avoid error timespec struct type redefinition windows headers 
    // (C:\Program Files (x86)\Windows Kits\10\Include\10.0.10240.0\ucrt\time.h(39)) vs. tine (at least v4.6.0)
    #endif

#endif

#ifdef SYSTEM_WINDOWS_VS2019_X64
    #define NOMINMAX // for usefulfunctions.cpp std::numeric_limits<float>::max() and ::min() (from 
        // #include <limits>), otherwise
        // collides with c:\Program Files (x86)\Windows Kits\8.1\Include\shared\minwindef.h 
        // definition of macros min and max)

    #include <stdio.h>

    #define WIN32_LEAN_AND_MEAN  // SW Dec 21, 2015 added so that winsock does not get included by windows.h (will collide with tine v4.5+ (year 2015) project.def
    // which may include winsock2 headers (IPV6)
    #include <windows.h>
    #include <WinSock2.h>    // SW Dec 21, 2015 added 
    #include <conio.h>
    #include <time.h>
    #include <assert.h>
    #include <sys\stat.h>
    #include <sys\timeb.h>

    #define __STDC_FORMAT_MACROS
    #include <inttypes.h>

    //#pragma warning(disable : 4996) // disable deprecated warnings SW commented Oct 15, 2019

    #define  SW_PLATFORM_STRING "win64(msvs2019,x64)"

    #define  PLF_TCPIPSTACK_VS2015_OR_GREATER // for deprecated gethostbyname -> getaddrinfo
    #define  PLF_WINNT_TCPIPSTACK

    #define  PLF_PATH_SEPARATOR_CHAR '\\'
    #define  PLF_PATH_SEPARATOR_CHAR_AS_STRING "\\"	
    #define  PLF_ALLFILES_WILDCARD	"*.*"	
    #define  platformvsnprintf _vsnprintf
    //	#define  platformsnprintf _snprintf
    // platformstrcasecmp case insensitive string comparison
    #define  platformstrcasecmp _stricmp
    #define  platformstrncasecmp _strnicmp // only compare N (rightmost argument) characters (insensitive match of some left part of string)

    #define  PLF_MAX_FDSETS 		FD_SETSIZE

    #define PLF_LITTLE_ENDIAN

    #define HAVE_TIMESPEC // SW Jan 5, 2018 for tine srvcore.h to avoid error timespec struct type redefinition windows headers 
    // (C:\Program Files (x86)\Windows Kits\10\Include\10.0.10240.0\ucrt\time.h(39)) vs. tine (at least v4.6.0)

#endif



#if defined(PLF_SYSTEM_LINUX)

    typedef    bool    BOOL;

//	#ifndef INT32
//		typedef		int				INT32;
//	#endif

    #ifndef UINT32
        typedef		unsigned int	UINT32;
    #endif

//	#ifndef DOUBLE
//		typedef		double			DOUBLE;
//	#endif

    #ifndef INT8
        typedef		signed char		INT8;
    #endif

    #ifndef UINT8
        typedef		unsigned char	UINT8; 
    #endif

    #ifndef UINT16
        typedef		unsigned short	UINT16;
    #endif

    #ifndef INT16
        typedef		signed short	INT16;
    #endif

    #ifndef INT64
        typedef		long long			INT64;
    #endif

    #ifndef UINT64
        typedef		unsigned long long UINT64;
    #endif

    #ifdef socktype
        #error socktype already defined! see Types.h  
    #else
        typedef		int				socktype;
    #endif

    #ifndef FLOAT
        typedef		float			FLOAT;
    #endif

    #ifndef HANDLE
        typedef		void *	HANDLE;
    #endif

    typedef void * PLF_SW_THREADRETURN;
    #define PLF_SW_THREADMARKER
    typedef void * PLF_SW_THREADPARAMETER;
    #define PLF_SW_THREADRETURN_VALUE NULL
    
    typedef struct {
        UINT64 time;
        UINT16 millitm; 
    } PLF_TIME;
    

#endif



#if defined(PLF_SYSTEM_WIN2015_X86) || defined(PLF_SYSTEM_WIN2019_X86)
//	#ifndef INT32
//		typedef		int				INT32;
//	#endif

    #ifndef UINT32
        typedef		unsigned int	UINT32;
    #endif

//	#ifndef DOUBLE
//		typedef		double			DOUBLE;
//	#endif

    #ifndef INT8
        typedef		signed char		INT8;
    #endif

    #ifndef UINT8
        typedef		unsigned char	UINT8; 
    #endif

    #ifndef UINT16
        typedef		unsigned short	UINT16;
    #endif

    #ifndef INT16
        typedef		signed short	INT16;
    #endif

    #ifndef INT64
        typedef		__int64			INT64;
    #endif

    #ifndef UINT64
        typedef		unsigned __int64 UINT64;
    #endif

    #ifdef socktype
        #error socktype already defined! see Types.h  
    #else
        typedef		int				socktype;
    #endif

    #ifdef socklen_t
        #error SW: for win32 platform socklen_t should not be defined!
    #else
        typedef		int				socklen_t;
    #endif

    typedef DWORD PLF_SW_THREADRETURN;
    #define PLF_SW_THREADMARKER WINAPI
    typedef LPVOID PLF_SW_THREADPARAMETER;
    #define PLF_SW_THREADRETURN_VALUE 0

    typedef struct {
        UINT32 time;
        UINT32 millitm; // make it 32bit for pairing/aligning
    } PLF_TIME;

    
#endif

#if defined(PLF_SYSTEM_WIN2015_X64) || defined(PLF_SYSTEM_WIN2019_X64)
    
    #ifndef INT32
        typedef		int				INT32;
    #endif

    #ifndef UINT32
        typedef		unsigned int	UINT32;
    #endif

    #ifndef DOUBLE
        typedef		double			DOUBLE;
    #endif

    #ifndef INT8
        typedef		signed char		INT8;
    #endif

    #ifndef UINT8
        typedef		unsigned char	UINT8; 
    #endif

    #ifndef UINT16
        typedef		unsigned short	UINT16;
    #endif

    #ifndef INT16
        typedef		signed short	INT16;
    #endif

    #ifndef INT64
        typedef		__int64			INT64;
    #endif

    #ifndef UINT64
        typedef		unsigned __int64 UINT64;
    #endif

    #ifdef socktype
        #error socktype already defined! see Types.h  
    #else
        typedef		SOCKET			socktype;
    #endif

    #ifdef socklen_t
        #error SW: for win64 platform socklen_t should not be defined!
    #else
        typedef		int				socklen_t;
    #endif

    typedef DWORD PLF_SW_THREADRETURN;
    #define PLF_SW_THREADMARKER WINAPI
    typedef LPVOID PLF_SW_THREADPARAMETER;
    #define PLF_SW_THREADRETURN_VALUE 0

    typedef struct {
        INT64 time;
        UINT16 millitm; // allowed value: 0..999 
    } PLF_TIME;
#endif 

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
    

/**	
    \brief Platform-independent type definition of thread startup function.
    
    Type definition in a platform-independent manner for a thread which can be started.
    Tested at least for posix style and for Win32 API.
    
    \remark This is pure internal type and subject to change if other platforms require change.
*/ 
typedef PLF_SW_THREADRETURN (PLF_SW_THREADMARKER *PLF_SW_THREAD_START_ROUTINE)(
    PLF_SW_THREADPARAMETER aThreadParameter
    );


/**	\brief Creates a counting semaphore with initial and maximum count settings.
    
    \remark This function must support binary semaphore (0,1) or (1,1) also.

    \param initial_count	The initial count of this semaphore, must be less than or equal to maximum count
    \param maximum_count	The maximum count of this semaphore, must be less than or equal max(UINT16)
    \return proper HANDLE in success or NULL in case of error
 */
extern HANDLE	platformCreateSemaphore( UINT16 initial_count, UINT16 maximum_count );

/** \brief Deletes a previously created counting semaphore.
 
    \param semaph	The semaphore handle to be deleted (must not be used after true return)
    \retval true successful deletion
    \retval false unsuccessful deletion or error case
 */ 
extern bool		platformDeleteSemaphore( HANDLE semaph );

/**
    \brief Waits for semaphore 'free' in order to take it (decrement by 1 if not 0).

    \param semaph			The semaphore handle to be obtained (must not be NULL)
    \param timeout_msec		Timeout in milliseconds before giving up waiting if semaphore is already taken.
    \retval	true successful obtaining of Semaphore
    \retval false unsuccessful obtain or error
 */ 
extern bool		platformWaitForSemaphore( HANDLE semaph, UINT32 timeout_msec );

/**
    \brief Releases/signals semaphore count (increment by 1 if not max).

    \param semaphore	The semaphore handle to be released (must not be NULL)
    \retval true successful release
    \retval false unsuccessful release or error 
 */ 
extern bool		platformReleaseSemaphore( HANDLE semaphore );

/**
    \brief Probes a semaphore for being possible to take (count > 0).
    
    \param semaph		The semaphore handle to be probed (must not be NULL)
    \retval	true successful probing
    \retval false unsuccessful or error case
 */ 
extern bool		platformIsSemaphoreSignalled( HANDLE semaph ) ;

/**	\brief Wrapper for non-signalling sleep function.

    Pauses the calling thread by a discrete amount of time. 

    \param msec		The time in milliseconds to pause
    \remark Take special care on e.g. UNIX platforms so that signals do not improperly interrupt the sleeping.
 */ 
extern void		platformSleepMilliseconds(UINT32 msec); 

/** \brief Creates, initializes and starts up a thread (passed as arguments).

    \param	aThreadFunction	The thread function defined somewhere in your code.
    \param	aThreadParameter	The parameter will be passed to the thread. Very useful for passing context information.
    \return proper HANDLE in success or NULL in case of error
 */ 
extern HANDLE	platformCreateThread( PLF_SW_THREAD_START_ROUTINE aThreadFunction, void *aThreadParameter); 


/** \brief Creates, initializes and starts up a critical (time priority) thread (passed as arguments).

    \param	aThreadFunction	The thread function defined somewhere in your code.
    \param	aThreadParameter	The parameter will be passed to the thread. Very useful for passing context information.
    \param  abShutdownThread	will internally be set to true in order to signal thread to shut down (optional parameter, may be null but beware null is dangerous)
    \param	abThreadRunning		will internally be read in order to get thread execution status (optional parameter, may be null but beware null is very dangerous)
    \param  abDelayThreadStart  will internally be set to false in order to let thread start to run (in order to exit) (optional parameter)
    \return proper HANDLE in success or NULL in case of error
 */ 
//extern HANDLE	platformCreateCriticalThread( PLF_SW_THREAD_START_ROUTINE aThreadFunction, void *aThreadParameter, volatile bool *abShutdownThread, volatile bool *abThreadRunning, volatile bool *abDelayThreadStart);
// commented(removed) platformCreateCriticalThread due to cleanup (unused function) 2022-06-08 

/**
    \brief Deletes a previously created thread HANDLE.
    
      The thread must be gracefully shut down before this function is called. It will clean up memory and 
      possibly unregisters it from the API / OS.

    \param aThreadHandle		thread handle to be cleaned up / deleted
    \retval	true success
    \retval false unsuccessful, error case
 */ 
extern bool		platformDeleteThread(HANDLE aThreadHandle);

/** \brief Gets the current time of the system clock and puts it into aTimeStruct
    
    \param aTimeStruct		on true return code, the current time will be put in there
    \retval	true success
    \retval false returned in error case

    \remark On false return, the parameter aTimeStruct is left untouched!
 */ 
extern bool		platformTimeGet( PLF_TIME &aTimeUTC);

// -1	a < b	a earlier b, b later a
// 0	a == b	a is exactly the same time than b
// 1	a > b	a is later than b, b is earlier than a
extern int platformTimeCompare( const PLF_TIME &a, const PLF_TIME &b);

// returns seconds (before fraction) and milliseconds (fractional part)
// return: 
//  ~0 times are identical
//  <0 aTimePast is future of aTimeCurrent
//  >0 aTimePast is past of aTimeCurrent
extern double platformTimeDelta( const PLF_TIME &aTimePast, const PLF_TIME &aTimeCurrent);


extern void platformTimeAddOffset( PLF_TIME &a, unsigned int offset_msec);

// 
// aTimeStamp - input: timestamp (UTC) which should be converted to human readable string
// apOutStringBuffer - on return, will be filled with aTimeStamp decoded to human readable timestamp format; if NULL, function does nothing
// aiStringBufferLengthBytes - needs to pass length of StringBuffer (recommendation is to use not less than 28 characters(bytes))
//
// return value: apOutStringBuffer (for convenient usage at printf-style functions)
//
// e.g. returned string (without single quotes):
//  '[time_conversion_error]'
//  '[28.03.16 14:28:59.312 LTM]'
extern char *platformTimeToString( const PLF_TIME &aTimeStampUTC, char *apOutStringBuffer, unsigned int aiStringBufferLengthBytes); // SW introduced Jun 20, 2016

// 
// aTimeStamp - input: timestamp (UTC) which should be converted to human readable string
// apOutStringBuffer - on return, will be filled with aTimeStamp decoded to human readable timestamp format; if NULL, function does nothing
// aiStringBufferLengthBytes - needs to pass length of StringBuffer (recommendation is to use not less than 60 characters(bytes))
//
// return value: apOutStringBuffer (for convenient usage at printf-style functions)
//
// e.g. returned string (without single quotes):
// '[time_conversion_error]'
// '[2022-03-28 14:28:59.312 LTM]' <- LTM is shown if time zone request failed.
// '[2022-03-14 14:28:59.312 CET]'
// '[2022-06-14 14:28:59.312 CEST]'
// '[2022-06-14 14:28:59.312 W. Europe Daylight Time]'
// '[2022-03-12 14:28:59.312 W. Europe Standard Time]'
extern char *platformTimeToStringEx(const PLF_TIME &aTimeStampUTC, char *apOutStringBuffer, unsigned int aiStringBufferLengthBytes); // SW introduced Jun 7, 2022

//asUTCString: "2022-03-16 12:46:22.254000 AM UTC" is March 16, 2022 00:46:22.254000 usec UTC
extern bool platformUTCStringToTimeT(const char *asUtcString, time_t &atOutUtcTime, uint32_t &anOutMicrosecondOffset); // SW introduced Jun 14, 2022

// 
 // aTimeStamp - input: timestamp (UTC) which should be converted to human readable string
 // apOutStringBuffer - on return, will be filled with aTimeStamp decoded to human readable 
 //                     timestamp format; if NULL, function does nothing
 // aiStringBufferLengthBytes - needs to pass length of StringBuffer (recommendation is to use not less than 36 characters(bytes))
 //
 // return value: apOutStringBuffer (for convenient usage at printf-style functions)
 //
 // e.g. returned string (without single quotes):
 //           111111111122222222223333333
 //  123456789012345678901234567890123456
 // '[2016-03-28 14:28:59.312 UTC]' 
 // '[2016-03-28 12:20:53.742 PM UTC]' 
 // '[2016-03-28  2:20:53.743262 PM UTC]' 
 // '[2016-03-28 14:20:53.743262 UTC]' 

 extern char *platformTimeToUTCString(time_t atUTCseconds,
     uint32_t anMilliOrMicroseconds,
     bool abMicroseconds,
     bool abWithBrackets,
     bool abUse12HourFormat,
     char *apOutStringBuffer,
     unsigned int aiStringBufferLengthBytes);

/** \brief Copies (clones) a file from aSourceFilename to aCloneFilenameToCreate.
    
    \param aSourceFilename			name of the source file (should include full path), must exist
    \param aCloneFilenameToCreate	name of the destionation (clone) file, may or must not exist
    \param aFailIfExist				fail in case the destination file (clone) does already exist, false: overwrite it!
    \retval	true successfully cloned file contents
    \retval false unsuccessful, error case
 */ 
extern bool 	platformFileCopy( const char *aSourceFilename, const char *aCloneFilenameToCreate, bool aFailIfExist);

/**	\brief Returns full path to executable file (including filename).

    \param aPathBuffer			path buffer memory location
    \param aMaxBufferLength		maximum length (in chars) of aPathBuffer NOT including mandatory nullbyte
    \retval	true successful put into argument buffer
    \retval false unsuccessful or error case
 */ 
extern 	bool platformGetFullExecutablePath( char *aPathBuffer, UINT32 aMaxBufferLength);


/**	\brief by definition, it removes single file only, not directory (uses sort of unlink command).

    \param aFilenamePath		filename (may include path) to the file
    \retval	true successfully deleted file
    \retval false unsuccessful or error case
 */ 
extern bool platformFileRemove( const char *aFilenamePath );

extern bool platformFileExists( const char *aFilenamePath );

// returns true in case of successful conversion, false otherwise
extern bool platformConvertStringToInt64(const char *apInputString, INT64 &aOutInt64Value);  // SW implemented Jul 8, 2014

//
// returns true in case of successful conversion, false otherwise
// returns false in case aStringBufferLength is too short!
// recommended aStringBufferLength for i64 is 21 characters (minus, 19 digits, \0)
extern bool platformConvertInt64ToString(INT64 aiValue, char *apOutStringBuffer, size_t aiStringBufferLength); // SW implemented Jul 8, 2014

////////////////////////////////////////////////////////////////////////
// socket function(s)
////////////////////////////////////////////////////////////////////////

extern 		void		platformInitializeSocketSubsystem();
extern 		void		platformDeinitializeSocketSubsystem();

extern		INT32		platformSocketGetLastError();
extern		socktype	platformSocketCreateInet( INT32 type );

extern		INT32		platformSocketReceiveFunction( socktype s, void *buf, UINT32 buflen);
extern		INT32		platformSocketSendFunction( socktype s, const void *buf, UINT32 buflen);

extern		INT32		platformSocketReceiveFromFunction( socktype s, void *buf, UINT32 buflen, UINT32 *ip_hostorder, UINT16 *port_hostorder);
extern		INT32		platformSocketSendToFunction( socktype s, const void *buf, UINT32 buflen, UINT32 ip_hostorder, UINT16 port_hostorder );

extern		INT32		platformSocketClose( socktype s);
extern		INT32		platformSocketConnect(socktype s, const struct sockaddr * Saddr, socklen_t size_sockaddr);
extern		INT32		platformSocketBind( socktype s, const struct sockaddr * Saddr, socklen_t size_sockaddr);
extern		INT32		platformSocketSetNonblocking ( socktype s );
extern		UINT32		platformIPAddrConvert( const char * const ip_addr_dotted_string);
extern 		char		*platformIPAddrConvert( UINT32 ip_hostorder );
extern		INT32		platformSocketDisconnectUDP( socktype s);
extern		INT32		platformSocketSelect(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
extern struct hostent*	platformSocketGetHostByName( char *hostname );
extern		INT32		platformSocketGetHostName( char *aOutHostName, unsigned int aHostNameBufferSizeElements );
extern char				*platformSocketGetOSErrorString( UINT32 intOSLastError, char *lasterrorstr, size_t length_string);
extern		bool		platformSocketIsBlockingError( UINT32 errcode );
extern const INT32		platformMaxFdSets; 

extern		INT32		platformSocketGetSockOpt(socktype s, int level, int optname, void *optval, socklen_t *optlen);
extern		INT32		platformSocketSetSockOpt(socktype s, int level, int optname, const void *optval, socklen_t optlen);

extern		INT16		platformHostToLittleEndian(INT16 aHost);
extern		INT16		platformHostToBigEndian(INT16 aHost);
extern		INT16		platformLittleEndianToHost(INT16 aLittleEndian);
extern		INT16		platformBigEndianToHost(INT16 aBigEndian);

extern		UINT16		platformHostToLittleEndian(UINT16 aHost);
extern		UINT16		platformHostToBigEndian(UINT16 aHost);
extern		UINT16		platformLittleEndianToHost(UINT16 aLittleEndian);
extern		UINT16		platformBigEndianToHost(UINT16 aBigEndian);

extern		INT32		platformHostToLittleEndian(INT32 aHost);
extern		INT32		platformHostToBigEndian(INT32 aHost);
extern		INT32		platformLittleEndianToHost(INT32 aLittleEndian);
extern		INT32		platformBigEndianToHost(INT32 aLittleEndian);

extern		UINT32		platformHostToLittleEndian(UINT32 aHost);
extern		UINT32		platformHostToBigEndian(UINT32 aHost);
extern		UINT32		platformLittleEndianToHost(UINT32 aLittleEndian);
extern		UINT32		platformBigEndianToHost(UINT32 aBigEndian);

extern		INT64		platformHostToLittleEndian(INT64 aHost);
extern		INT64		platformHostToBigEndian(INT64 aHost);
extern		INT64		platformLittleEndianToHost(INT64 aLittleEndian);
extern		INT64		platformBigEndianToHost(INT64 aBigEndian);

extern		UINT64		platformHostToLittleEndian(UINT64 aHost);
extern		UINT64		platformHostToBigEndian(UINT64 aHost);
extern		UINT64		platformLittleEndianToHost(UINT64 aLittleEndian);
extern		UINT64		platformBigEndianToHost(UINT64 aBigEndian);

extern		float		platformHostToLittleEndian(float aHost);
extern		float		platformHostToBigEndian(float aHost);
extern		float		platformLittleEndianToHost(float aLittleEndian);
extern		float		platformBigEndianToHost(float aBigEndian);

extern		double		platformHostToLittleEndian(double aHost);
extern		double		platformHostToBigEndian(double aHost);
extern		double		platformLittleEndianToHost(double aLittleEndian);
extern		double		platformBigEndianToHost(double aBigEndian);

//extern		bool		platformInitializeConsoleHandler();
//extern		bool		platformDeinitializeConsoleHandler();

extern void platformInitializeConsoleInput();
extern void platformDeinitializeConsoleInput();
extern INT32 platformkbhit();
extern INT32 platformgetch();

extern int64_t platformFileTell64(FILE *apFile); // introduced 2022-05-05
extern bool platformFileSeek64(FILE *apFile, int64_t ai64Offset, int aiOrigin); // introduced 2022-05-05
extern bool platformFileGetLastModifiedTime64(const char *asFilePath, PLF_TIME & aOutUTCTimeLastModified); // introduced 2022-06-07

// gets the host name of the computer we're on (linux: gethostname, windows: getcomputernameex)
// 
// Parameters:
//  asOutBuffer - memory buffer for hostname string that will be returned
//  aiInOutBufferSizeBytes - in: number of bytes asOutBuffer has available
//                         - out: number of bytes used (including nullbyte) (return value 1)
//                                number of bytes required (including nullbyte) (return value 0)
//                                any other return code: not modified
//
// Return Value:
//    1    success
//    0    buffer too small (aiInOutBufferSize returns minimal buffer size)
//   -1    error getting host name (not buffer too small)
//   -2    parameter error (asOutBuffer or aiInOutBufferSize are NULL)
//
// Remarks:
//  - in case of return code 0 (buffer too small), aiInOutBufferSize will contain required buffer size
//  - on linux, buffer size should be HOST_NAME_MAX+1 characters
// 
extern INT32 platformGetHostName( char *asOutBuffer, size_t *aiInOutBufferSizeBytes );


// Returns the current user name
//
// Return Values:
//   1    success
//   0    buffer too small
//   -1   error getting user name
//   -2   parameter error (asOutBuffer, aiInOutBufferSizeBytes must not be NULL)
//
// Remarks:
//  - for windows usually UNLEN=256, 256+1 bytes should be enough (for char=1 byte)
//  - for linux usually 32 characters are maximum, so 33 bytes should be enough
//
INT32 platformGetUserName( char *asOutBuffer, size_t *aiInOutBufferSizeBytes ) ;

// Returns the current process id and also retrieves the name (full path+exe) for this process id
//
// Return Values:
//   1    success
//   0    buffer too small
//   -1   error getting process id
//   -2   parameter error (asOutBuffer, aiInOutBufferSizeBytes must not be NULL)
//   -3   process id too big (does not fit UINT32)
//
// Remarks:
//
//  - even in case of buffer too small, asOutBuffer may be altered!
//  - linux will truncate the full path of the process to the buffer size possible
//  - recommended buffer size is 1024 bytes
//
INT32 platformGetProcessNameID( char *asOutBuffer, size_t *aiInOutBufferSizeBytes, UINT32 *anOutProcessID);

// Returns the default logfile path per OS
// (windows: getenv("TEMP))
// (linux/solaris: "/tmp" )
// (global default: "")

// Return Values:
//   1    success
//   0    buffer too small
//   -1   parameter error (asOutBuffer, aiInOutBufferSizeBytes must not be NULL)
//
// Remarks:
//  - recommended buffer size is 1024 bytes
//  - default log file path may or may not have a path separator char appended!
INT32 platformGetDefaultLogFilePath(char *asOutBuffer, size_t *aiInOutBufferSizeBytes);

#endif //VIDEOSYSTEM3_PLATFORM_H_EXTERNAL

#endif // __PLATFORM_H_INCLUDED_SW__
