#ifndef __VIDEOSYSTEM3_H_INCLUDED_SW__
#define __VIDEOSYSTEM3_H_INCLUDED_SW__

// SW Jul 28, 2014 NOTE start:
//  videosystem3.h must only be used by client projects which use videosystem3 shared library
//  for internal use in videosystem3, a special define DEFINE_VIDEOSYSTEM3_H_INTERNAL must be declared prior to
//  include videosystem3.h, otherwise platform.h will only declare minimal amount of symbols
// SW Jul 28, 2014 NOTE end

#ifndef DEFINE_VIDEOSYSTEM3_H_INTERNAL
#define VIDEOSYSTEM3_PLATFORM_H_EXTERNAL
#endif

// win32 dll header file (also include for client sourcecode which uses this dll)

#include "system.h"
#include <videosystem3/platform.h>  // vsv3 include, directory '...\general\src\' 

/* TINE_DECORATED_CONSTANTS removed Jul 22, 2019 (TINE Release4 upgrade to TINE Release 5)
#ifndef TINE_DECORATED_CONSTANTS
 //
 // SW Nov 29, 2012 1/6 Inside videosystem3 library, TINE constants are  
 // SW Nov 29, 2012 2/6 prepended with TDC_ to avoid naming collisions.
 // SW Nov 29, 2012 3/6 Before Nov 29, 2012, user developer had to define 
 // SW Nov 29, 2912 4/6 TINE_DECORATED_CONSTANTS by oneself but after Nov 29, 
 // SW Nov 29, 2012 5/6 2012 this is done inside here so user developer does
 // SW Nov 29, 2012 6/6 not need to care about.
 // 
 #define TINE_DECORATED_CONSTANTS
#endif
*/
#include <videosystem3/v3clientlib-tineincludewrapper.h>





//////////////////////////////////////////////////////////////////////////////////////////////////

// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the VIDEOSYSTEM3_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// VIDEOSYSTEM3_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
//
// SW Dec 7, 2011 extension:
//
// there are now THREE possibilities
//
// 1. the header is used for integrating library code directly into user's project
//    - in this case neither VIDEOSYSTEM3_DLL_EXPORTS nor VIDEOSYSTEM3_DLL_IMPORTS must be defined!
// 2. the header is used for creating the DLL
//    - VIDEOSYSTEM3_DLL_EXPORTS needs to be defined
//    - VIDEOSYSTEM3_DLL_IMPORTS must not be defined
// 3. the header is used at user's project for declaring functions used FROM the dll
//    - VIDEOSYSTEM3_DLL_IMPORTS needs to be defined
//    - VIDEOSYSTEM3_DLL_EXPORTS must not be defined

#if defined(PLF_SYSTEM_WINNT) || defined(PLF_SYSTEM_WIN2015) || defined(PLF_SYSTEM_WIN2019)
    // the following branch is only for windows platforms
    #if defined(VIDEOSYSTEM3_DLL_EXPORTS) && defined(VIDEOSYSTEM3_DLL_IMPORTS)
        #error VIDEOSYSTEM3_DLL_EXPORTS and VIDEOSYSTEM3_DLL_IMPORTS must not be defined at the same time
    #endif

    #if defined(VIDEOSYSTEM3_DLL_EXPORTS)
        #define VIDEOSYSTEM3_API __declspec(dllexport)
    #elif defined (VIDEOSYSTEM3_DLL_IMPORTS)
        #define VIDEOSYSTEM3_API __declspec(dllimport)
    #else
        // define to 'nothing'
        #define VIDEOSYSTEM3_API 
    #endif
#elif defined(PLF_SYSTEM_LINUX)
    // SW Dec 7, 2011: on any other platform (just Linux now) we are only using gcc, so 'extern' (required for 
    // shared library) is default (i read this in a forum post) so the only thing left to do for any of the three
    // choices (shlib export, shlib import, direct code integration) is define VIDEOSYSTEM3_API to 'nothing'

    #define VIDEOSYSTEM3_API 
#else
    #error VIDEOSYSTEM3_API define needs to be configured for this platform!
#endif



// v2core.h defines VIDINFHDR structure
// well, structures are in general bad to pass from code to dll and vice versa, but for sake of simplicity it is done
// here that way. Attention needs to be payed to struct member alignment (msvc6: Project -> Settings -> 
// C/C++ -> Code Generation -> struct member alignment needs to be set to default (8 bytes))
#include <videosystem3/v2core.h>
#include <videosystem3/v2algorithms.h>
#include <videosystem3/v2videofileload.h>
#include <videosystem3/v3clientapi.h>

#endif // __VIDEOSYSTEM3_H_INCLUDED_SW__

